package com.zhiLian.datasource;


import com.zhiLian.datasource.config.factory.DruidDataSourceFactory;
import com.zhiLian.datasource.config.role.DynamicDataSource;
import com.zhiLian.datasource.domain.DataSourceInfo;
import com.zhiLian.datasource.domain.EntInfo;
import lombok.extern.log4j.Log4j2;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author DongZl
 * @description: 数据源配置
 * @Date 2023-8-1 上午 11:05
 */
@Log4j2
@Configuration
public class ManyDataSource {


    private List<EntInfo> dataSourceInfoList(){
        List<EntInfo> databaseNameList = new ArrayList<>(){{
            add(EntInfo.builder()
                    .entCode("test_1")
                    .ip("192.168.120.128")
                    .port(3306)
                    .build());
            add(EntInfo.builder()
                    .entCode("test_2")
                    .ip("122.51.111.225")
                    .port(6666)
                    .build());
            add(EntInfo.builder()
                    .entCode("test_3")
                    .ip("122.51.111.225")
                    .port(3333)
                    .build());
        }};
        return databaseNameList;
    }



    @Bean
    @Primary
    public DynamicDataSource dynamicDataSource(DruidDataSourceFactory druidDataSourceFactory) {

        //查企业
        Map<Object, Object> dataSourceMap = new HashMap<>();
        dataSourceInfoList()
                .stream()
                .map(entInfo-> DataSourceInfo.hostAndPortBuild(entInfo.getEntCode(),entInfo.getIp(),entInfo.getPort()))
                .forEach(dataSourceInfo -> {
            dataSourceMap.put(dataSourceInfo.getKey(), druidDataSourceFactory.create(dataSourceInfo));
        });
        //设置动态数据源
        DynamicDataSource dynamicDataSource = new DynamicDataSource();
//        dynamicDataSource.setDefaultTargetDataSource(masterDataSource());
        dynamicDataSource.setTargetDataSources(dataSourceMap);
        //将数据源信息备份在defineTargetDataSources中
        dynamicDataSource.setDefineTargetDataSources(dataSourceMap);
        return dynamicDataSource;
    }
}
