package com.zhiLian.iotdb.consumer;

import lombok.extern.log4j.Log4j2;
import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.stereotype.Component;

import java.time.Duration;
import java.util.Collections;
import java.util.Properties;

/**
 * @ClassName:
 * @Description:
 * @Author: zhuwenqiang
 * @Date: 2023/11/2
 */
@Component
@Log4j2
public class HelloWorldKafkaConsumer {

    @KafkaListener(topics = { "biwe-topic" })
    public void helloWorldKafkaConsumer(ConsumerRecord<String, String> record1) {
//        String key = record1.key();
//        String value = record1.value();

        // 创建 properties 对象 配置 kafka消费者的配置信息
        Properties properties = new Properties();
        properties.setProperty(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG, "111.229.102.61:9092");
        // 设置 键值的反序列化方式
        properties.setProperty(ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG, StringDeserializer.class.getName());
        properties.setProperty(ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG, StringDeserializer.class.getName());
        // 配置 分组  ***
        properties.setProperty(ConsumerConfig.GROUP_ID_CONFIG, "bw2");

        // 创建 kafka 消息消费者对象
        KafkaConsumer<String, String> kafkaConsumer = new KafkaConsumer<String, String>(properties);
        // 订阅主题
        kafkaConsumer.subscribe(Collections.singleton("bwie-topic"));
        while (true) {
            // 拉取消息
            ConsumerRecords<String, String> records = kafkaConsumer.poll(Duration.ofMillis(2000));
            // 遍历
            records.forEach(record -> {
                String key = record.key();
                String value = record.value();
                System.out.println("消息者消息成功，消息的key：" + key + "，value：" + value);
            });
        }
//        log.info("消息的消费者接收到消息，消息的key：{}，value：{}", key, value);
    }

    @KafkaListener(topics = {"topic1", "topic2"},
            containerFactory = "kafkaListenerContainerFactory",
            errorHandler = "myKafkaListenerErrorHandler")
    public void helloWorldKafkaConsumer(ConsumerRecord<String, String> record,
                                        Acknowledgment acknowledgment) {
        String key = record.key();
        String value = record.value();
        log.info("消息的消费者接收到消息，消息的key：{}，value：{}", key, value);
        // 手动确认
        acknowledgment.acknowledge();
    }


}
